# inspired from:
# https://github.com/pypa/warehouse/blob/master/warehouse/__about__.py

__all__ = [
    '__title__', '__summary__', '__uri__', '__version__',
    '__author__', '__email__', '__license__', '__copyright__',
]

__title__ = 'sagalogger'
__summary__ = 'A json log formatter.'
__uri__ = 'https://www.github.com/Sagacify/logger-py'

__version__ = '0.3.2'


__author__ = 'Sagacify'
__email__ = 'dev+python@sagacify.com'

__license__ = 'MIT'
__copyright__ = '2016 %s' % __author__



"""
Logging configuration utils.

Contains saga formatter and function to create a logger.
"""
from .__about__ import (
    __author__, __copyright__, __email__, __license__, __summary__,
    __title__, __uri__, __version__
)
from .formatter import SagaFormatter
from .creator import get_logger

__all__ = [
    'SagaFormatter', 'get_logger', '__author__', '__copyright__', '__email__',
    '__license__', '__summary__', '__title__', '__uri__', '__version__']



import logging
import os
import sys
from functools import partial
from .formatter import SagaFormatter

# Bunyan log levels are slightly different cfr:
# https://docs.python.org/2/library/logging.html#levels
LOG_LEVELS = {
    'fatal': 50,
    'error': 40,
    'warn': 30,
    'info': 20,
    'debug': 10,
    'trace': 0,
}

LOG_HANDLER = logging.StreamHandler(stream=sys.stdout)
FORMATTER = SagaFormatter()
LOG_HANDLER.setFormatter(FORMATTER)

LOG_LEVEL_NAME = os.environ.get('LOG_LEVEL')

if LOG_LEVEL_NAME is None:
    LOG_LEVEL = LOG_LEVELS['info']
else:
    LOG_LEVEL = LOG_LEVELS[LOG_LEVEL_NAME.lower()]

if LOG_LEVEL == LOG_LEVELS['trace']:
    ROOT_LOGGER = logging.getLogger()
    ROOT_LOGGER.addHandler(LOG_HANDLER)
    ROOT_LOGGER.setLevel(LOG_LEVEL)


def get_logger(module):
    logger = logging.getLogger(module)
    logger.addHandler(LOG_HANDLER)
    logger.setLevel(LOG_LEVEL)

    def log(event, data=None, meta=None, level=0):
        if not isinstance(event, str):
            event = event.__repr__()
        if data is not None and isinstance(data, str):
            data = {'message': data}
        if meta is not None and isinstance(meta, str):
            meta = {'message': meta}
        logger.log(
            level,
            {
                'event': event,
                'data': data,
                'meta': meta
            }
        )

    for name, level in LOG_LEVELS.items():
        logger.__setattr__(name, partial(log, level=level))

    return logger



"""
SagaFormatter

Ensures logs are formatted as we want them.
cfr: https://github.com/sagacify/logger
"""
import datetime
import json
import os
import socket
import sys
import time
import traceback


def error_serializer(error):
    """Serialize error as bunyan.

    When in doubt, ask what would bunyan do?
    https://github.com/trentm/node-bunyan/blob/master/lib/bunyan.js#L1141
    """
    trace = traceback.format_stack(error.__traceback__)
    return {
        'message': ' '.join(error.args),
        'name': error.__class__.__name__,
        'stack': [line for line in trace]
    }


def extra_serializer(obj):
    if isinstance(obj, datetime.datetime):
        result = obj.isoformat() + 'Z'
    elif isinstance(obj, datetime.date):
        result = obj.isoformat()
    elif isinstance(obj, datetime.time):
        result = obj.strftime('%H:%M')
    elif isinstance(obj, BaseException):
        result = error_serializer(obj)
    else:
        result = {'str': str(obj)}
    return result


def get_name_from_args():
    return (sys.argv[0] or sys.executable).split(os.sep)[-1]


def get_app_name_version():
    main_module = sys.modules['__main__']

    # If importing from jupyter hub or cli.
    if main_module.__package__ is None:
        return get_name_from_args(), sys.version.split(' ')[0]

    app_package = sys.modules[main_module.__package__]
    try:
        app_name = app_package.__title__
        app_version = app_package.__version__
    except AttributeError:
        print('FATAL: could not access package info.')
        sys.exit(-1)
    return app_name, app_version


def format_time(record):
    """Format time to ISO 8601.

    https://en.wikipedia.org/wiki/ISO_8601
    """
    utc_time = time.gmtime(record.created)
    time_string = time.strftime('%Y-%m-%d %H:%M:%S', utc_time)
    return '%s.%03dZ' % (time_string, record.msecs)


class SagaFormatter(object):
    """
    Saga Formatter.

    Formats mesages coming from a sagalogger
    """

    def __init__(self):
        self.app_name, self.app_version = get_app_name_version()
        self.hostname = socket.gethostname()

    def format(self, record):
        """
        Formats a log record and serializes to json
        """
        if isinstance(record.msg, dict):
            event = record.msg.get('event')
            data = record.msg.get('data')
            meta = record.msg.get('meta')
            msg = ''
        else:
            msg = str(record.msg)
            event = None
            data = None
            meta = None

        return json.dumps({
            'name': self.app_name,
            'version': self.app_version,
            'module': record.name or record.module,
            'time': format_time(record),
            'v': 0,
            'hostname': self.hostname,
            'pid': record.process,
            'level': record.levelno + 10,
            'msg': msg,
            'event': event,
            'data': data,
            'meta': meta
            }, default=extra_serializer)



import datetime
import json
import logging
import unittest
from io import StringIO
import src as sagalogger


class TestJsonLogger(unittest.TestCase):
    def setUp(self):
        self.logBuffer = StringIO()

        self.logger = logging.getLogger('sagalogger-test')
        self.logger.setLevel(logging.DEBUG)

        self.logHandler = logging.StreamHandler(self.logBuffer)
        self.logger.addHandler(self.logHandler)

    def testDefaultFormat(self):
        fr = sagalogger.SagaFormatter()
        self.logHandler.setFormatter(fr)

        msg = 'testing logging format'
        self.logger.info(msg)
        logJson = json.loads(self.logBuffer.getvalue())

        self.assertEqual(logJson['msg'], msg)

    def testFormatKeys(self):
        supported_keys = [
            'hostname',
            'level',
            'module',
            'msg',
            'name',
            'pid',
            'time',
            'v'
        ]
        fr = sagalogger.SagaFormatter()
        self.logHandler.setFormatter(fr)

        msg = 'testing sagalogger'
        self.logger.info(msg)
        log_msg = self.logBuffer.getvalue()
        log_json = json.loads(log_msg)

        for supported_key in supported_keys:
            self.assertIn(supported_key, log_json)

    def testLogAnEvent(self):
        fr = sagalogger.SagaFormatter()

        self.logHandler.setFormatter(fr)

        msg = {
            'event': 'PONEY_START',
            'data': 'test',
            'meta': {'url': 'meta_test'}
        }

        self.logger.info(msg)
        logJson = json.loads(self.logBuffer.getvalue())

        self.assertEqual(logJson.get('event'), msg['event'])
        self.assertEqual(logJson.get('data'), msg['data'])
        self.assertEqual(logJson.get('meta'), msg['meta'])

    def testJsonDateEncoder(self):
        fr = sagalogger.SagaFormatter()

        self.logHandler.setFormatter(fr)

        data = {
            'one': datetime.datetime(1999, 12, 31, 23, 59),
            'two': datetime.datetime(1900, 1, 1)
        }

        self.logger.info({'event': 'PROUT', 'data': data})
        logJson = json.loads(self.logBuffer.getvalue())

        self.assertEqual(logJson['data'].get('one'), '1999-12-31T23:59:00Z')
        self.assertEqual(logJson['data'].get('two'), '1900-01-01T00:00:00Z')

    def testJsonErrorEncoder(self):
        fr = sagalogger.SagaFormatter()

        self.logHandler.setFormatter(fr)

        data = Exception('The world is harsh!')

        self.logger.info({'event': 'PROUT', 'data': data})
        logJson = json.loads(self.logBuffer.getvalue())
        print(logJson['data'])

        self.assertEqual(logJson['data']['message'], 'The world is harsh!')
        self.assertEqual(logJson['data']['name'], 'Exception')
        self.assertIs(type(logJson['data']['stack']), list)



from setuptools import setup
from io import open
import os

root = os.path.abspath(os.path.dirname(__file__))


with open(os.path.join(root, 'src', '__about__.py'), encoding='utf8') as f:
    about = {}
    exec(f.read(), about)


rst_file = os.path.join(root, 'README.rst')
if not os.path.isfile(rst_file):
    with open(rst_file, 'w', encoding='utf-8') as readme:
        import pypandoc
        readme.write(pypandoc.convert(os.path.join(root, 'README.md'), 'rst'))


long_description = open(rst_file, encoding='utf-8').read()


setup(
    name=about['__title__'],
    packages=[about['__title__']],
    package_dir={about['__title__']: 'src'},
    description=about['__summary__'],
    long_description=long_description,
    author=about['__author__'],
    author_email=about['__email__'],
    version=about['__version__'],
    url=about['__uri__'],
    license=about['__license__']
)
